@@ -3,16 +3,16 @@ from __future__ import annotations
 import asyncio
 import concurrent.futures
 import functools
-import multiprocessing
+import multiprocessing.queues
 import queue
 import threading
 import weakref
 from abc import ABC, abstractmethod
 from collections.abc import Hashable
 from datetime import datetime
-from typing import Optional
 
-from .util import MAX_THREADS
+from .concurrent.futures import ThreadPoolExecutor
+from .threading import MAX_THREADS
 
 
 class Task:
@@ -169,18 +169,18 @@ class BackgroundTask(ABC):
     Facilities are provided to check task status, and cancel an ongoing task.
     """
 
-    def __init__(self, executor: Optional[concurrent.futures.Executor] = None):
+    def __init__(self, executor: concurrent.futures.Executor | None = None):
         """
         Parameters
         ----------
         executor
             If you provide your own *process* executor,
-            it's recommended to use ``mpservice.util.MP_SPAWN_CTX`` for its
+            it's recommended to use ``mpservice.multiprocessing.MP_SPAWN_CTX`` for its
             ``ctx`` parameter.
         """
         self._own_executor = False
         if executor is None:
-            executor = concurrent.futures.ThreadPoolExecutor(MAX_THREADS)
+            executor = ThreadPoolExecutor(MAX_THREADS)
             self._own_executor = True
         self._executor = executor
         self._tasks: dict[Hashable, Task] = {}
@@ -198,7 +198,7 @@ class BackgroundTask(ABC):
         cls,
         *args,
         _cancelled: threading.Event,
-        _info: queue.Queue | multiprocessing.Queue,
+        _info: queue.Queue | multiprocessing.queues.Queue,
         **kwargs,
     ):
         """
@@ -319,7 +319,7 @@ class BackgroundTask(ABC):
     def __getitem__(self, task_id: str) -> Task:
         return self._tasks[task_id]
 
-    def get(self, task_id: str, default=None) -> Optional[Task]:
+    def get(self, task_id: str, default=None) -> Task | None:
         return self._tasks.get(task_id, default)
 
     def __iter__(self):
