@@ -2,31 +2,66 @@ from __future__ import annotations
 
 import asyncio
 import concurrent.futures
+import functools
+import inspect
 import logging
 import os
 import queue
 import stat
+import subprocess
 import threading
 import time
 from collections.abc import Iterable, Sequence
 from pickle import dumps as pickle_dumps
 from pickle import loads as pickle_loads
 from time import perf_counter
-from typing import Any, Awaitable, Callable, Optional
-
-from overrides import EnforceOverrides
+from typing import Any, Awaitable, Callable
 
 from ._queues import SingleLane
-from .util import (
-    MAX_THREADS,
-    RemoteException,
-    get_docker_host_ip,
-    is_async,
-)
+from .concurrent.futures import ThreadPoolExecutor
+from .multiprocessing import RemoteException
+from .threading import MAX_THREADS
+
+__all__ = [
+    'SocketApplication',
+    'SocketServer',
+    'SocketClient',
+    'make_server',
+]
 
 logger = logging.getLogger(__name__)
 
 
+def is_async(func):
+    while isinstance(func, functools.partial):
+        func = func.func
+    return inspect.iscoroutinefunction(func) or (
+        not inspect.isfunction(func)
+        and hasattr(func, "__call__")
+        and inspect.iscoroutinefunction(func.__call__)
+    )
+
+
+def get_docker_host_ip():
+    """
+    From within a Docker container, this function finds the IP address
+    of the host machine.
+    """
+    # INTERNAL_HOST_IP=$(ip route show default | awk '/default/ {print $3})')
+    # another idea:
+    # ip -4 route list match 0/0 | cut -d' ' -f3
+    #
+    # Usually the result is '172.17.0.1'
+    #
+    # The command `ip` is provided by the Linux package `iproute2`.
+
+    z = subprocess.check_output(
+        ["ip", "-4", "route", "list", "match", "0/0"]  # noqa: S603, S607
+    )
+    z = z.decode()[len("default via ") :]
+    return z[: z.find(" ")]
+
+
 def put_in_queue(q, x, stop_event, timeout=0.1):
     """
     ``q`` is either a ``threading.Queue`` or a ``multiprocessing.queues.Queue``,
@@ -73,7 +108,8 @@ def encode(data, encoder):
         return pickle_dumps(data)
     if encoder == "utf8":
         return data.encode("utf8")
-    assert encoder == "none"
+    if encoder != "none":
+        raise ValueError(f"expecting 'none' but got: {encoder}")
     return data  # `data` must be bytes
 
 
@@ -186,7 +222,7 @@ async def open_unix_connection(path: str, *, timeout=None):
             await asyncio.sleep(0.1)
 
 
-class SocketApplication(EnforceOverrides):
+class SocketApplication:
     """
     ``SocketApplication`` is designed to to used similar to the "application" in a
     HTTP framework. The main API is to register "endpoint" functions by the method
@@ -209,8 +245,8 @@ class SocketApplication(EnforceOverrides):
     def __init__(
         self,
         *,
-        on_startup: Optional[Sequence[Callable]] = None,
-        on_shutdown: Optional[Sequence[Callable]] = None,
+        on_startup: Sequence[Callable] | None = None,
+        on_shutdown: Sequence[Callable] | None = None,
     ):
         self.on_startup = on_startup or []
         self.on_shutdown = on_shutdown or []
@@ -226,9 +262,9 @@ class SocketApplication(EnforceOverrides):
         and returns a response (which could be ``None`` if so desired).
         The response should be serializable by the encoder.
         To be safe, return a object of Python native types.
-        If exception is raised in this method, appropriate :class:`~mpservice.util.RemoteException`
+        If exception is raised in this method, appropriate :class:`RemoteException`
         object will be sent in the response.
-        The method could also proactively return a :class:`~mpservice.util.RemoteException` object.
+        The method could also proactively return a :class:`RemoteException` object.
 
         ``path`` is any string. The route is identified by this string. For familiarity,
         it may be a good idea to start the string with ``'/'``, although this is in no
@@ -248,15 +284,15 @@ class SocketApplication(EnforceOverrides):
         return await self._routes[path](data)
 
 
-class SocketServer(EnforceOverrides):
+class SocketServer:
     def __init__(
         self,
         app: SocketApplication,
         *,
-        path: Optional[str] = None,
-        host: Optional[str] = None,
-        port: Optional[int] = None,
-        backlog: Optional[int] = None,
+        path: str | None = None,
+        host: str | None = None,
+        port: int | None = None,
+        backlog: int | None = None,
         shutdown_path: str = "/shutdown",
     ):
         """
@@ -299,7 +335,7 @@ class SocketServer(EnforceOverrides):
     def __str__(self):
         return self.__repr__()
 
-    async def run(self):
+    async def serve(self):
         """
         Start the server and let it stay up until shutdown conditions are met.
         """
@@ -356,6 +392,7 @@ class SocketServer(EnforceOverrides):
         async def _keep_receiving():
             # Infinite loop to handle requests on this connection
             # until the connection is closed by the client.
+            loop = asyncio.get_running_loop()
             while True:
                 try:
                     req_id, data = await read_record(reader, timeout=0.1)
@@ -367,7 +404,7 @@ class SocketServer(EnforceOverrides):
                 path = data[0]
                 data = data[1]
                 if path == self._shutdown_path:
-                    t = asyncio.Future()
+                    t = loop.create_future()
                     await reqs.put((req_id, t))
                     self.to_shutdown = True
                     t.set_result(None)
@@ -417,23 +454,14 @@ def make_server(app: SocketApplication, **kwargs):
     return SocketServer(app, **kwargs)
 
 
-def run_app(app, **kwargs):
-    """
-    End user typically calls this function to start the server.
-    The server will stay up until the client requests its shutdown.
-    """
-    server = make_server(app, **kwargs)
-    asyncio.run(server.run())
-
-
-class SocketClient(EnforceOverrides):
+class SocketClient:
     def __init__(
         self,
         *,
-        path: Optional[str] = None,
-        host: Optional[str] = None,
-        port: Optional[int] = None,
-        num_connections: Optional[int] = None,
+        path: None | str = None,
+        host: None | str = None,
+        port: None | int = None,
+        num_connections: None | int = None,
         connection_timeout: int = 60,
         backlog: int = 2048,
     ):
@@ -481,9 +509,9 @@ class SocketClient(EnforceOverrides):
         self._encoder = "pickle"  # encoder when sending requests.
         self._prepare_shutdown = threading.Event()
         self._to_shutdown = threading.Event()
-        self._executor = concurrent.futures.ThreadPoolExecutor()
+        self._executor = ThreadPoolExecutor()
         self._tasks = []
-        self._pending_requests: Optional[queue.Queue] = None
+        self._pending_requests: queue.Queue | None = None
         self._active_requests = {}
         self._shutdown_timeout = 60
 
@@ -512,11 +540,6 @@ class SocketClient(EnforceOverrides):
         logger.info("client %s is ready", self)
         return self
 
-    # def __exit__(self, exc_type=None, exc_value=None, exc_traceback=None):
-    # msg = exit_err_msg(self, exc_type, exc_value, exc_traceback)
-    # if msg:
-    #     logger.error(msg)
-
     def __exit__(self, *args, **kwargs):
         self._prepare_shutdown.set()
         t0 = perf_counter()
@@ -704,7 +727,10 @@ class SocketClient(EnforceOverrides):
                 if t.done():
                     if t.exception():
                         raise t.exception()
-                    assert self._to_shutdown.is_set()
+                    if not self._to_shutdown.is_set():
+                        raise ValueError(
+                            f"expecting `self._to_shutdown.is_set()` to be True but got: {self._to_shutdown.is_set()}"
+                        )
                     break
                 if self._to_shutdown.is_set():
                     break
